#include <gmp.h>
#include "flint.h"
#include "ulong_extras.h"
#include "fmpz.h"

#ifdef POPCNT_INTRINSICS
static __inline__ mp_bitcnt_t shortCount(slong val)
{
#if defined(_WIN64) || defined(__mips64)
   return __builtin_popcountll(val);  
#else
   return __builtin_popcountl(val);
#endif
}
#else
/* A naive implementation if neither your processor nor your compiler want to
 * do the work. */
static __inline__ mp_bitcnt_t shortCount(slong val)
{
        mp_bitcnt_t cnt;
        for(cnt=0; val; val >>= 1) {
                cnt += val & 1;
        }
        return cnt;
}
#endif

#ifdef __x86_64__
#include <cpuid.h>

static mp_bitcnt_t slow_fmpz_popcnt(const fmpz_t c)
{
        fmpz c1;
        c1 = *c;
        if(!COEFF_IS_MPZ(c1))
        {
                if(*c < 0) return 0;
                else return __builtin_popcountl(*c);
        } else
        {
                __mpz_struct *t = COEFF_TO_PTR(c1);
                if(flint_mpz_cmp_si(t,0) < 0) return 0;
                else return mpz_popcount(t);
        }
}

static mp_bitcnt_t __attribute__((target ("popcnt")))
fast_fmpz_popcnt(const fmpz_t c)
{
        fmpz c1;
        c1 = *c;
        if(!COEFF_IS_MPZ(c1))
        {
                if(*c < 0) return 0;
                else return __builtin_popcountl(*c);
        } else
        {
                __mpz_struct *t = COEFF_TO_PTR(c1);
                if(flint_mpz_cmp_si(t,0) < 0) return 0;
                else return mpz_popcount(t);
        }
}

static mp_bitcnt_t (*resolve_fmpz_popcnt(void))(const fmpz_t c) {
        unsigned int eax, ebx, ecx, edx;
        return (__get_cpuid(1, &eax, &ebx, &ecx, &edx) &&
                (ecx & bit_POPCNT) != 0)
                ? &fast_fmpz_popcnt
                : &slow_fmpz_popcnt;
}

mp_bitcnt_t __attribute__((ifunc ("resolve_fmpz_popcnt")))
fmpz_popcnt(const fmpz_t c);
#else
mp_bitcnt_t fmpz_popcnt(const fmpz_t c)
{
        fmpz c1;
        c1 = *c;
        if(!COEFF_IS_MPZ(c1))
        {
                if(*c < 0) return 0;
                else return shortCount(*c);
        } else
        {
                __mpz_struct *t = COEFF_TO_PTR(c1);
		if(flint_mpz_cmp_si(t,0) < 0) return 0;
                else return mpz_popcount(t);
        }
}
#endif
